
import QuartzCore
import Security

open class Foo {}

extension FixedWidthInteger where Self: UnsignedInteger {

  /// Creates a value by mapping `source` from the full range of its type
  /// `S` to the full range of `Self`.
  init<S>(rangeConverted source: S)
  where S: FixedWidthInteger, S: UnsignedInteger {
    let srcByteCount = MemoryLayout<S>.size
    let dstByteCount = MemoryLayout<Self>.size
    if srcByteCount < dstByteCount {
      let dstBitCount = dstByteCount &* 8
      self = Self(truncatingIfNeeded: source)
      var shifts = srcByteCount &* 8
      while shifts < dstBitCount {
        self |= self &<< shifts
        shifts = shifts &* 2
      }
    } else if srcByteCount > dstByteCount {
      let shifts = (srcByteCount &- dstByteCount) &* 8
      let a = source &>> shifts
      self = Self(truncatingIfNeeded: a)
    } else {
      self = Self(truncatingIfNeeded: source)
    }
  }
}

func demonstrateStrangeWayOfSpeedingUpCode() {
  let num = 100_000_000
  var randomBytes = [UInt8](repeating: 0, count: num)
  for trial in 0 ..< 5 {

    print("#\(trial)")

    // Set new random bytes for each trial:
    randomBytes.withUnsafeMutableBytes { (ptr) -> Void in
      let rc = SecRandomCopyBytes(nil, ptr.count, ptr.baseAddress!)
      precondition(rc == errSecSuccess)
    }

    // ---- The Normal Test ----
    var checksum = UInt64(0)
    let t0 = CACurrentMediaTime()
    var i = 0
    let count = randomBytes.count
    while i < count {
      let dst = UInt64(rangeConverted: randomBytes[i])
      checksum = checksum ^ dst
      i = i &+ 1
    }
    let t1 = CACurrentMediaTime()
    print("  Normal Test:", t1 - t0, "seconds (checksum: \(checksum))")

    // ---- The Faster Test ----
    fasterTestBySimplyCopyPastingTheAboveNormalTestInsideMe2(randomBytes)
  }
}

@inline(never)
func fasterTestBySimplyCopyPastingTheAboveNormalTestInsideMe2(_ randomBytes: [UInt8]) {
  var checksum = UInt64(0)
  let t0 = CACurrentMediaTime()
  var i = 0
  let count = randomBytes.count
  while i < count {
    let dst = UInt64(rangeConverted: randomBytes[i])
    checksum = checksum ^ dst
    i = i &+ 1
  }
  let t1 = CACurrentMediaTime()
  print("  Faster Test2:", t1 - t0, "seconds (checksum: \(checksum))")
}

demonstrateStrangeWayOfSpeedingUpCode()
